/** 
 *   FIXME ....
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 **/
#define _GNU_SOURCE

#include <stdio.h>

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>

//#include "e5850.h"
#include "encoders.h"
#include "my_stdio.h"
#include "iar.h"
//#include "coordinate.h"
//#include "anemometer.h"
//#include "antenna.h"
#include "socket_client.h"
#include "iar_bus.h"
#include "antenna_ib_pk.h"

int encoder_num[] = {RA_CASE, DEC_CASE};
//char antenna_hostname[_POSIX_PATH_MAX] = {ANTENNA_HOST_NAME};

//struct encoders_data *d = NULL;

/*****************************************************************************
 * local functions:
 **/


static int encoders_send_cmd(int fd, int cmd)
{
	struct ib_packet pk;
	struct antenna_packet *anp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	anp = (struct antenna_packet *)&pk.data;
	l = sizeof(struct antenna_packet_head);
	anp->h.cmd = cmd;
	anp->h.size = 0;
	ibus_send_data_to(fd, ENCODERS_BUS_ADDR, anp, l);

	ret = ibus_read(fd, &pk);
	if( ret > 0) {
		ret = atoi(anp->data);
		PVERB(PINFO,"# val ret: %d\n", ret);
	}

	return ret;

}


static int encoder_n_send_cmd(int fd, int num, int cmd)
{
	struct ib_packet pk;
	struct antenna_packet *anp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	anp = (struct antenna_packet *)&pk.data;
	l = sizeof(struct antenna_packet_head);
	anp->h.size = sprintf(anp->data,"%d", num);
	l += anp->h.size;
	anp->h.cmd = cmd;
	ibus_send_data_to(fd, ENCODERS_BUS_ADDR, anp, l);

	ret = ibus_read(fd, &pk);
	if( ret > 0) {
		ret = atoi(anp->data);
		PVERB(PINFO,"# val ret: %d\n", ret);
	}

	return ret;

}


/*
 * local functions END.
 *****************************************************************************/

int encoders_open(int ra, int dec)
{
	int fd;

	encoder_num[RA_CASE] = ra;
        encoder_num[DEC_CASE] = dec;
	fd = ANTENNA_CALL_SOCKET();

	return fd;
}


int encoders_close(int fd)
{
	ibus_remove( fd );

	return close(fd);
}


int encoders_buff_len(int fd)
{
	return encoders_send_cmd(fd, ECM_GETBUFFLEN);
}


int encoders_clear_buff(int fd)
{     
	/* FIXME: */
	return encoders_send_cmd(fd, ECM_CLEARBUFF);
}


int encoders_is_running(int fd)
{
	/* FIXME: */
	return encoders_send_cmd(fd, ECM_ISRUNNING);
}


int encoders_stop(int fd)
{
	return ibus_off(fd);
	//return encoders_send_cmd(fd, ECM_STOP);
}

	
int encoders_start(int fd)
{
	return ibus_on(fd);
	//return encoders_send_cmd(fd, ECM_START);
}



int encoders_get_next_data(int fd, struct encoders_data *en)
{
	struct ib_packet pk;
	struct antenna_packet *ap;
	struct encoders_data *d;

	ap = (struct antenna_packet *)&pk.data;
	d = (struct encoders_data *)ap->data;
	ibus_read(fd, &pk);

	if (ap->h.cmd == ECM_POSITION) {
		memcpy(en, d, sizeof(struct encoders_data));
		return 0;
	}
	if (ap->h.cmd == SCM_INFO) {
		printf("%s\n", (char *) ap->data);
		return SCM_INFO;
	}
	if (ap->h.cmd == WCM_WIND) {
		//ap->data[ap->h.size] = 0;
		//printf("dwt: %s\n", (char *) ap->data);
		return WCM_WIND;
	}

	PVERB(PWARN,"encoders_get_next_data: unknow pk cmd 0x%x\n", 
	      ap->h.cmd  );

	return -1;
}


int get_encoder(int fd, int num)
{
	return encoder_n_send_cmd(fd, num, ECM_GETNVAL);
}


int encoder_set_cw(int fd, int num)
{
	return encoder_n_send_cmd(fd, num, ECM_SETCW);
}


int encoder_set_ccw(int fd, int num)
{
	return encoder_n_send_cmd(fd, num, ECM_SETCCW);
}


int encoder_set_zero(int fd, int num)
{
	return encoder_n_send_cmd(fd, num, ECM_SETZERO);
}


int encoder_set_toggle_zero(int fd, int num)
{
	return encoder_n_send_cmd(fd, num, ECM_TOGGLEZERO);
}


int sidereal_clock_running(int fd)
{
	return encoders_send_cmd(fd, ECM_SCLKRUNNING);
}


int put_on_fail_condition(int fd)
{
	return encoders_send_cmd(fd, ECM_FAILMODE);
}


int put_on_normal_condition(int fd)
{
	return encoders_send_cmd(fd, ECM_NORMALMODE);
}


int get_anemometer(int fd)
{
	return encoder_n_send_cmd(fd, 2, ECM_GETNVAL);
}

