/** 
 *   FIXME ....
 *
 *   Survey for "antenna" position system.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ??? 
 **/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <malloc.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/time.h>

//#include "iar.h"
#include "antenna_ib_pk.h"
#include "iar_bus.h"
#include "my_stdio.h"
#include "my_math.h"
#include "survey.h"
#include "encoders.h"
#include "coordinate.h"
#include "anemometer.h"

#define TIME_SL 17 /* length of timestamp in register */

static int start_survey = 0;
static int fd_save = -1;
static char *file_save = NULL;
static int ra, dec;
static 	int done = 0;

struct wt_info {
	char *data;
	int size;
	struct timeval tv;
};

static struct wt_info wt = {NULL,0,{0,0}};


static int antenna_send_info(int fd, const char *fmt, ...)
{
        va_list ap;
        int l;
        struct antenna_packet apk;

        va_start(ap, fmt);
        apk.h.size = vsnprintf (apk.data, ANTENA_PACKET_DATA_LEN, fmt, ap);
        va_end(ap);

	l = apk.h.size + sizeof(struct antenna_packet_head);
        apk.h.cmd = SCM_INFO;
        ibus_send_data( fd, &apk, l);

        return l;
}


static int push_wt_data(char *data, int len)
{
	gettimeofday(&wt.tv, NULL);
	if (wt.data == NULL) {
		wt.data = (char *) malloc((len + 1) * sizeof(char));
		wt.size = len;
	}
	if (wt.size < len) {
		free(wt.data);
		wt.data = NULL;
		wt.size = 0;
		push_wt_data(data, len);
	} else {
		memset(wt.data, 0, wt.size+1);
		memcpy(wt.data, data, len);
		//*(wt.data+len) = 0;
	}

	return 0;
}


static int open_save_file(char *buff)
{
	/* file_save */
	int fd_s;
	char data_dir[] = {"/var/antennad"};
	char *np = buff;
	char *endp;
	int alfa_norm, delta_norm;
	int grid = 5;
  	time_t now;
	struct tm *t;

	if( file_save == NULL)
		file_save = (char *) malloc(_POSIX_PATH_MAX * sizeof(char));
	/* skip timestamp: */
	strtod(np, &endp );
	np = endp + 1;

	alfa_norm =grid_norm(strtod(np, &endp ), grid);
	np = endp + 1;
	delta_norm =grid_norm(strtod(np, &endp ), grid);
	/* Create date stamp and open data file */
	time(&now);
	t = localtime(&now);
	sprintf(file_save, "%s/%04d%02d%02d_%03d_%03d.dat", 
		data_dir, 
		t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
		alfa_norm, delta_norm );
	//sprintf(file_save, "MM.dat");
	fd_s = open(file_save, O_WRONLY|O_CREAT|O_APPEND/*|O_SYNC*/,
		       S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (fd_s >= 0) {
		/* print header */
		fd_printf(fd_s,"# Antenna 2 position survey\n");
		fd_printf(fd_s,"# %04d/%02d/%02d %02d:%02d:%02d\n",
			  t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
			  t->tm_hour, t->tm_min, t->tm_sec);
		fd_printf(fd_s,"# pos: %03d %03d\n#\n", 
			  alfa_norm, delta_norm );
	}

	return fd_s;
}


static int make_reg(char *buff, struct encoders_data *d, struct wt_info *w)
{
	int l;
	int wt_status = 0;
	struct timeval t;

	gettimeofday(&t, NULL);
	if( w->tv.tv_sec + WT_EXPIRATION < t.tv_sec)
		wt_status = 1;

	l = sprintf(buff, "%ld.%06ld %8.4f %8.4f %1d %s", 
		    t.tv_sec, t.tv_usec,
		    ENC2DEG(d->val[ra]), ENC2DEC(d->val[dec]),
		    wt_status, w->data
		);
	/*
	l = sprintf(buff, "%ld.%06ld", t.tv_sec, t.tv_usec);
	l += sprintf(buff+l, " %8.4f %8.4f", 
		     ENC2DEG(d->val[ra]), 
		     ENC2DEC(d->val[dec])
		);
	l += sprintf(buff+l, " %1d %s\n", wt_status, w->data);
	*/
	buff[l] = 0;
	
	return l;
}


static int manager_track(int fd)
{
	char buff[160];
	static char buff_old[160];
	static int data_skip = 0;
	int reg_len;
	int l;
	struct ib_packet pk;
	struct antenna_packet *ap;
	struct encoders_data *d;
	int send_ret;
	int ret;

	send_ret = 0;
	ap = (struct antenna_packet *)&pk.data;
	d = (struct encoders_data *)ap->data;
	if ((ret = ibus_read(fd, &pk)) <= 0) {
		done = 1;
		return -2;
	}
	ret = 0;
	switch (ap->h.cmd) {
	case ECM_POSITION:
		reg_len = make_reg(buff, d, &wt);
		PVERB(PINFO, "%s\n", buff); 
		if( start_survey ) {
			if (fd_save < 0) {
				/* init file */
				/* FIXME */
				fd_save = open_save_file(buff);
				if (fd_save < 0) {
					perror("open save file:");
					antenna_send_info(fd,  
							 "Error opening: %s", 
							 file_save);
					SYSLOG(LOG_ERR,"Error opening: %s", 
					       file_save);
					start_survey = 0;
				}else {
					antenna_send_info(fd,  
							 "Save data in: %s", 
							 file_save);
				}
			}
			if ( fd_save > 0) {
				if (strncmp(buff+TIME_SL, buff_old+TIME_SL,
					    reg_len - TIME_SL - 3) != 0) { 
					/* 3 is for wind degrees */
					if(data_skip > 0)
						fd_printf(fd_save,"%s\n", 
							  buff_old);
					fd_printf(fd_save,"%s\n", buff);
					data_skip = 0;
				} else {
					data_skip++;
				}
				memcpy(buff_old, buff, reg_len);
			}
		}
		break;
	case SCM_SURVEY_START:
		start_survey = 1;
		send_ret = 1;
		ret = 0;
		//ibus_on( fd );
		break;
	case SCM_SURVEY_STOP:
		start_survey = 0;
		if ( fd_save > 0) {
			antenna_send_info(fd, "Close file: %s", file_save);
			start_survey = 0;
			close(fd_save);
			free(file_save);
			file_save = NULL;
			fd_save = -1;
		}
		send_ret = 1;
		ret = 0;
		//ibus_off( fd );
		break;
	case SCM_SURVEY_STATUS:
		if ( start_survey ) {
			ap->h.size = sprintf(ap->data, 
					     "Save in file: %s", file_save);
		}else{
			ap->h.size = sprintf(ap->data, "Stoped");
		}
		ap->h.cmd = SCM_INFO;
		l =  ap->h.size + sizeof(struct antenna_packet_head);
		ibus_send_data_to( fd, pk.h.addr, ap, l);
		PVERB(PINFO,"# Sending: [%s]\n", ap->data);
		break;
	default:
		antenna_send_info(fd, "Survey: unknown cmd 0x%x", ap->h.cmd);
		PVERB(PINFO,"# Survey: unknown cmd 0x%x from: 0x%x\n",
		      ap->h.cmd, pk.h.addr);
		send_ret = 1;
		ret = -1;
		break;
	case WCM_WIND:
		/* FIXME: Implementarlo !!! */
		push_wt_data(ap->data, ap->h.size);
		send_ret = 0;
		PVERB(PINFO, "# wt info: %ld -> [%s], size: %d\n", 
		      wt.tv.tv_sec, wt.data, wt.size);
		break;
	}
	if ( send_ret) {
		/* Send ret value: */
		PVERB(PINFO,"# sendind ret: %d\n", ret);
		l = sizeof(struct antenna_packet_head);
		l += sprintf(ap->data, "%d",  ret);
		ap->h.cmd = CM_RETURN;
		ibus_send_data_to(fd, pk.h.addr, ap, l);
	}

	return ret;
}


int survey_core(int fd)
{
	fd_set rfds;
	struct timeval tv;
	int retval;

	//ra  = encoder_num[RA_CASE];
	//dec = encoder_num[DEC_CASE];
	ra  = RA_CASE;
	dec = DEC_CASE;

	//if (start_survey) 
	ibus_on( fd );

	done = 0;
	while( !done ) {
		FD_ZERO(&rfds);
		FD_SET(fd, &rfds);
		tv.tv_sec = 10;
		tv.tv_usec = 0;
		retval = select(fd + 1, &rfds, NULL, NULL, &tv);
		if (retval) {
			if (FD_ISSET(fd, &rfds)) {
				manager_track(fd);
			}
		}
	}
	if (fd_save > 0) {
		close(fd_save);
		free(file_save);
		file_save = NULL;
		fd_save = -1;
	}

	return 0;
}
