/** 
 *   Survey daemon
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 *      *) validation (getpwent? lh_stats ?).
 *      *) Options at startup.
 **/

#define _GNU_SOURCE

#include <errno.h> 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
//#include <sys/resource.h>
#include <sys/wait.h>
//#include <sys/socket.h>
#include <sys/file.h>
#include <limits.h>

#include "socket_srv.h"
#include "socket_client.h"
#include "antenna_ib_pk.h"
#include "my_stdio.h"
#include "iar_bus.h"
#include "survey.h"


void intHandler (int signo) 
{
	SYSLOG(LOG_ERR, "Bus: write error, SIGPIPE trapped\n");
	return;
}


static void usage(void)
{
	static const char *msg[] =
	{
	"Survey: antenna position system.",
	"",
	"Version: ""VERSION",
	"http://www.iar.unlp.edu.ar/~fede/pub/posicionador",
	"",
	"(c) Copyright 2004, 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,",
	"Instituto Argentino de Radio Astronomia (IAR).",
	"",
	"Usage:",
	"surveyd [-vln]",
	" -v\t\t - Verbose (incremental).",
	" -n, --nodaemon\t - No run as a daemon.",
	" -l, --nolog\t - No write events to syslog.",
	""
	};
	unsigned j;
	for (j = 0; j < sizeof(msg)/sizeof(msg[0]); j++)
		fprintf(stderr, "\t%s\n", msg[j]);


	return;
}


void close_log(void)
{
        SYSLOG(LOG_INFO, "Program terminated.");
}



int main(int argc, char *argv[]) 
{
	int fd;
	int log;
	char *program_name;
#ifndef NODAEMON
        int pid;
        int fd_pid;
        char pid_file[_POSIX_PATH_MAX];
#endif
	char *cp;
	int op;
	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 'h'},
		{"verbose", 0, 0, 'v'},
		{"nodaemon", 0, 0, 'n'},
		{"nolog", 0, 0, 'l'},
		{0,  0,  0,  0}
	};
	int daemon_flag = 1;
	
	syslog_flag = 1;
	/* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];

	opterr =  0;
	while  ((op = getopt_long(argc, argv, "vhnl",longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
#ifndef NODAEMON
		case 'n':
			daemon_flag = 0;
			break;
#endif
		case 'l':
			syslog_flag = 0;
			break;
		case 'h':
			usage();
			//help();
			exit(-1); 
		default:
			usage();
			exit(-1);
		}
#       ifndef NODAEMON
	if(daemon_flag){
		/* Ok, a daemon */
                verbose_flag = 0;
                if ( (pid=fork()) != 0 ) {
                        sprintf(pid_file, "/var/run/%s.pid", program_name);
                        fd_pid = open(pid_file, O_WRONLY|O_CREAT|O_SYNC,
                                      S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
			if (fd_pid > 0) {
				fd_printf(fd_pid,"%d\n", pid);
				close(fd_pid);
			}else{
				perror("Can't open pid file");
				fprintf(stderr, "%s\n", pid_file);
			}
                        exit(0);
                }
        }
#       endif
	if(syslog_flag){
		openlog(program_name, LOG_PID, LOG_DAEMON );
		atexit(close_log);
		SYSLOG(LOG_INFO, "Starting %s.", program_name);
	}
	if (signal(SIGPIPE, intHandler) == SIG_ERR) {
		perror("Signal Failed (SIGPIPE).");
		exit(1);
	}
	while(1) {
		/* survey: registry on bus */
		log = 1;
		while((fd = ANTENNA_CALL_SOCKET()) < 0) {
			if (log) 
				SYSLOG(LOG_WARNING, 
				       "Waiting connection to encoders.");
			log = 0;
			sleep(1);
		}
		if ( ibus_set_addr(fd, SURVEY_BUS_ADDR) != SURVEY_BUS_ADDR) {
			SYSLOG(LOG_ERR,"Survey bus addr falled.");
			exit(1);
		}
		SYSLOG(LOG_INFO,"Connected");
		survey_core(fd);
		//ibus_remove( fd );
		//close( fd );
	}

	return 0;
}


