/** 
 *   FIXME ....
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include "my_stdio.h"
#include "iar.h"
#include "socket_client.h"
#include "iar_bus.h"
#include "antenna_ib_pk.h"
#include "dwt.h"


/*****************************************************************************
 * local functions:
 **/


static int dwt_send_cmd(int fd, int cmd)
{
	struct ib_packet pk;
	struct antenna_packet *anp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	anp = (struct antenna_packet *)&pk.data;
	l = sizeof(struct antenna_packet_head);
	anp->h.cmd = cmd;
	anp->h.size = 0;
	ibus_send_data_to(fd, DWT_BUS_ADDR, anp, l);

	ret = ibus_read(fd, &pk);
	if( ret > 0) {
		ret = atoi(anp->data);
		PVERB(PINFO,"# val ret: %d\n", ret);
	}

	return ret;

}


static int get_data_from_dwt(int fd, int cmd, char *data)
{
	struct ib_packet pk;
	struct antenna_packet *anp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	anp = (struct antenna_packet *)&pk.data;
	l = sizeof(struct antenna_packet_head);
	anp->h.cmd = cmd;
	anp->h.size = 0;
	ibus_send_data_to(fd, DWT_BUS_ADDR, anp, l);

	ret = ibus_read(fd, &pk);
	if( ret > 0) {
		memcpy(data, anp->data, anp->h.size);
		ret = ibus_read(fd, &pk);
	}

	return ret;
}



/*
 * local functions END.
 *****************************************************************************/


int open_dwt(char *host)
{
	if (host[0] != 0) {
		sprintf(dwt_hostname, "%s", host);
	}
	printf("# caling: %s on port %d\n", dwt_hostname, DWT_TCP_PORT);
	return  call_socket(dwt_hostname, DWT_TCP_PORT);
}

int close_dwt(int fd)
{
	ibus_remove( fd );

	return close(fd);
}


char *GET_loop(int fd)
{
	char *loop = (char *) malloc(LOOP_LEN * sizeof(char));

	if (get_data_from_dwt(fd, WCM_LOOP, loop) < 0) {
		free(loop);
		loop = NULL;
	}

	return loop;
}

ushort wake_recv(int fd)
{
	/* FIXME: */
	return 0;
}

int CMD_settime(int fd)
{
	return dwt_send_cmd(fd, WCM_SETTIME);
}


struct tm *CMD_gettime(int fd)
{ 
	struct tm *tp = (struct tm *) malloc(sizeof(struct tm));

	if (get_data_from_dwt(fd, WCM_GETTIME, (char *) tp) < 0) {
		free(tp);
		tp = NULL;
	}

	return tp;
}


char *CMD_test(int fd)
{
	char *msg = (char *) malloc(8 * sizeof(char));

	if (get_data_from_dwt(fd, WCM_TEST, (char *) msg) < 0) {
		free(msg);
		msg = NULL;
	}

	return msg;
}


char *CMD_rxcheck(int fd)
{
	char *msg = (char *) malloc(35 * sizeof(char));
	
	if (get_data_from_dwt(fd, WCM_RXCHECK, (char *) msg) < 0) {
		free(msg);
		msg = NULL;
	}

	return msg;
}


char *CMD_ver(int fd)
{
	char *msg = (char *) malloc(18 * sizeof(char));
	
	if (get_data_from_dwt(fd, WCM_VER, (char *) msg) < 0) {
		free(msg);
		msg = NULL;
	}

	return msg;
}


int CMD_wrd(int fd)
{
	return dwt_send_cmd(fd, WCM_WRD);
}

